@@ -50,6 +50,7 @@
 #include <sys/time.h>
 #include <time.h>
 #include <unistd.h>
+#include <limits.h>
 
 #include <linux/can.h>
 #include <linux/can/error.h>
@@ -119,7 +120,7 @@ void calc_tv(struct timeval *tv, struct timeval *read_tv,
 		tv->tv_usec += read_tv->tv_usec;
 	}
 
-	if (tv->tv_usec > 1000000) {
+	if (tv->tv_usec >= 1000000) {
 		tv->tv_usec -= 1000000;
 		tv->tv_sec++;
 	}
@@ -131,74 +132,15 @@ void eval_can(char* buf, struct timeval *date_tvp, char timestamps, char base, i
 	static struct timeval tv; /* current frame timestamp */
 	static struct timeval read_tv; /* frame timestamp from ASC file */
 	struct canfd_frame cf;
+	struct can_frame *ccf = (struct can_frame *)&cf; /* for len8_dlc */
 	char rtr;
 	int dlc = 0;
+	int len = 0;
 	int data[8];
 	char tmp1[BUFLEN];
 	char dir[3]; /* 'Rx' or 'Tx' plus terminating zero */
 	char *extra_info;
-	int i, items, found;
-
-	/* 0.002367 1 390x Rx d 8 17 00 14 00 C0 00 08 00 */
-
-	found = 0; /* found valid CAN frame ? */
-
-	if (base == 'h') { /* check for CAN frames with hexadecimal values */
-
-		items = sscanf(buf, "%lu.%lu %d %s %2s %c %d %x %x %x %x %x %x %x %x",
-			       &read_tv.tv_sec, &read_tv.tv_usec, &interface,
-			       tmp1, dir, &rtr, &dlc,
-			       &data[0], &data[1], &data[2], &data[3],
-			       &data[4], &data[5], &data[6], &data[7]);
-
-		if ((items == dlc + 7 ) || /* data frame */
-		    ((items == 6) && (rtr == 'r')) || /* RTR without DLC */
-		    ((items == 7) && (rtr == 'r'))) { /* RTR with DLC */
-			found = 1;
-			get_can_id(&cf, tmp1, 16);
-		}
-
-	} else { /* check for CAN frames with decimal values */
-
-		items = sscanf(buf, "%lu.%lu %d %s %2s %c %d %d %d %d %d %d %d %d %d",
-			       &read_tv.tv_sec, &read_tv.tv_usec, &interface,
-			       tmp1, dir, &rtr, &dlc,
-			       &data[0], &data[1], &data[2], &data[3],
-			       &data[4], &data[5], &data[6], &data[7]);
-
-		if ((items == dlc + 7 ) || /* data frame */
-		    ((items == 6) && (rtr == 'r')) || /* RTR without DLC */
-		    ((items == 7) && (rtr == 'r'))) { /* RTR with DLC */
-			found = 1;
-			get_can_id(&cf, tmp1, 10);
-		}
-	}
-
-	if (found) {
-
-		if (dlc > CAN_MAX_DLC)
-			return;
-
-		if (strlen(dir) != 2) /* "Rx" or "Tx" */
-			return;
-
-		if (dir[0] == 'R')
-			extra_info = " R\n";
-		else
-			extra_info = " T\n";
-
-		cf.len = dlc;
-		if (rtr == 'r')
-			cf.can_id |= CAN_RTR_FLAG;
-		else
-			for (i = 0; i < dlc; i++)
-				cf.data[i] = data[i] & 0xFFU;
-
-		calc_tv(&tv, &read_tv, date_tvp, timestamps, dplace);
-		prframe(outfile, &tv, interface, &cf, CAN_MAX_DLEN, extra_info);
-		fflush(outfile);
-		return;
-	}
+	int i, items;
 
 	/* check for ErrorFrames */
 	if (sscanf(buf, "%lu.%lu %d %s",
@@ -215,8 +157,79 @@ void eval_can(char* buf, struct timeval *date_tvp, char timestamps, char base, i
 			calc_tv(&tv, &read_tv, date_tvp, timestamps, dplace);
 			prframe(outfile, &tv, interface, &cf, CAN_MAX_DLEN, "\n");
 			fflush(outfile);
+			return;
 		}
 	}
+
+	/* 0.002367 1 390x Rx d 8 17 00 14 00 C0 00 08 00 */
+
+	/* check for CAN frames with (hexa)decimal values */
+	if (base == 'h')
+		items = sscanf(buf, "%lu.%lu %d %s %2s %c %x %x %x %x %x %x %x %x %x",
+			       &read_tv.tv_sec, &read_tv.tv_usec, &interface,
+			       tmp1, dir, &rtr, &dlc,
+			       &data[0], &data[1], &data[2], &data[3],
+			       &data[4], &data[5], &data[6], &data[7]);
+	else
+		items = sscanf(buf, "%lu.%lu %d %s %2s %c %x %d %d %d %d %d %d %d %d",
+			       &read_tv.tv_sec, &read_tv.tv_usec, &interface,
+			       tmp1, dir, &rtr, &dlc,
+			       &data[0], &data[1], &data[2], &data[3],
+			       &data[4], &data[5], &data[6], &data[7]);
+
+	if (items < 7 ) /* make sure we've read the dlc */
+		return;
+
+	/* dlc is one character hex value 0..F */
+	if (dlc > CAN_MAX_RAW_DLC)
+		return;
+
+	/* retrieve real data length */
+	if (dlc > CAN_MAX_DLC)
+		len = CAN_MAX_DLEN;
+	else
+		len = dlc;
+
+	if ((items == len + 7 ) || /* data frame */
+	    ((items == 6) && (rtr == 'r')) || /* RTR without DLC */
+	    ((items == 7) && (rtr == 'r'))) { /* RTR with DLC */
+
+		/* check for CAN ID with (hexa)decimal value */
+		if (base == 'h')
+			get_can_id(&cf, tmp1, 16);
+		else
+			get_can_id(&cf, tmp1, 10);
+
+		/* dlc > 8 => len == CAN_MAX_DLEN => fill len8_dlc value */
+		if (dlc > CAN_MAX_DLC)
+			ccf->len8_dlc = dlc;
+
+		if (strlen(dir) != 2) /* "Rx" or "Tx" */
+			return;
+
+		/* check for signed integer overflow */
+		if (dplace == 4 && read_tv.tv_usec >= INT_MAX / 100)
+			return;
+
+		if (dplace == 5 && read_tv.tv_usec >= INT_MAX / 10)
+			return;
+
+		if (dir[0] == 'R')
+			extra_info = " R\n";
+		else
+			extra_info = " T\n";
+
+		cf.len = len;
+		if (rtr == 'r')
+			cf.can_id |= CAN_RTR_FLAG;
+		else
+			for (i = 0; i < len; i++)
+				cf.data[i] = data[i] & 0xFFU;
+
+		calc_tv(&tv, &read_tv, date_tvp, timestamps, dplace);
+		prframe(outfile, &tv, interface, &cf, CAN_MAX_DLEN, extra_info);
+		fflush(outfile);
+	}
 }
 
 void eval_canfd(char* buf, struct timeval *date_tvp, char timestamps, int dplace, FILE *outfile) {
@@ -264,13 +277,21 @@ void eval_canfd(char* buf, struct timeval *date_tvp, char timestamps, int dplace
 	if (strlen(dir) != 2) /* "Rx" or "Tx" */
 		return;
 
+	/* check for signed integer overflow */
+	if (dplace == 4 && read_tv.tv_usec >= INT_MAX / 100)
+		return;
+
+	/* check for signed integer overflow */
+	if (dplace == 5 && read_tv.tv_usec >= INT_MAX / 10)
+		return;
+
 	if (dir[0] == 'R')
 		extra_info = " R\n";
 	else
 		extra_info = " T\n";
 
 	/* don't trust ASCII content - sanitize data length */
-	if (dlen != can_dlc2len(can_len2dlc(dlen)))
+	if (dlen != can_fd_dlc2len(can_fd_len2dlc(dlen)))
 		return;
 
 	get_can_id(&cf, tmp1, 16);
@@ -327,6 +348,15 @@ void eval_canfd(char* buf, struct timeval *date_tvp, char timestamps, int dplace
 			/* dlen is always 0 for classic CAN RTR frames
 			   but the DLC value is valid in RTR cases */
 			cf.len = dlc;
+			/* sanitize payload length value */
+			if (dlc > CAN_MAX_DLEN)
+				cf.len = CAN_MAX_DLEN;
+		}
+		/* check for extra DLC when having a Classic CAN with 8 bytes payload */
+		if ((cf.len == CAN_MAX_DLEN) && (dlc > CAN_MAX_DLEN) && (dlc <= CAN_MAX_RAW_DLC)) {
+			struct can_frame *ccf = (struct can_frame *)&cf;
+
+			ccf->len8_dlc = dlc;
 		}
 	}
 
