@@ -240,7 +240,7 @@ void print_usage(char *prg)
 	fprintf(stderr, "          -u <uid>  (user defined modification identifier)\n");
 	fprintf(stderr, "          -l <hops>  (limit the number of frame hops / routings)\n");
 	fprintf(stderr, "          -f <filter>  (set CAN filter)\n");
-	fprintf(stderr, "          -m <mod>  (set Classic CAN frame modifications)\n");
+	fprintf(stderr, "          -m <mod>  (set Classical CAN frame modifications)\n");
 	fprintf(stderr, "          -M <MOD>  (set CAN FD frame modifications)\n");
 	fprintf(stderr, "          -x <from_idx>:<to_idx>:<result_idx>:<init_xor_val>  (XOR checksum)\n");
 	fprintf(stderr, "          -c <from>:<to>:<result>:<init_val>:<xor_val>:<crctab[256]>  (CRC8 cs)\n");
@@ -251,12 +251,12 @@ void print_usage(char *prg)
 	fprintf(stderr, "  <can_id>:<can_mask>  (matches when <received_can_id> & mask == can_id & mask)\n");
 	fprintf(stderr, "  <can_id>~<can_mask>  (matches when <received_can_id> & mask != can_id & mask)\n");
 	fprintf(stderr, "\n");
-	fprintf(stderr, "<mod> is a CAN frame modification instruction consisting of\n");
+	fprintf(stderr, "<mod> is a Classical CAN frame modification instruction consisting of\n");
 	fprintf(stderr, "<instruction>:<can_frame-elements>:<can_id>.<can_dlc>.<can_data>\n");
 	fprintf(stderr, "  <instruction>  is one of 'AND' 'OR' 'XOR' 'SET'\n");
 	fprintf(stderr, "  <can_frame-elements>  is _one_ or _more_ of 'I'dentifier 'L'ength 'D'ata\n");
 	fprintf(stderr, "  <can_id>  is an u32 value containing the CAN Identifier\n");
-	fprintf(stderr, "  <can_dlc>  is an u8 value containing the data length code (0 .. 8)\n");
+	fprintf(stderr, "  <can_dlc>  is an u8 value containing the data length code in hex (0 .. F)\n");
 	fprintf(stderr, "  <can_data>  is always eight(!) u8 values containing the CAN frames data\n");
 	fprintf(stderr, "\n");
 	fprintf(stderr, "<MOD> is a CAN FD frame modification instruction consisting of\n");
@@ -265,7 +265,7 @@ void print_usage(char *prg)
 	fprintf(stderr, "  <canfd_frame-elements>  is _one_ or _more_ of 'I'd 'F'lags 'L'ength 'D'ata\n");
 	fprintf(stderr, "  <can_id>  is an u32 value containing the CAN FD Identifier\n");
 	fprintf(stderr, "  <flags>  is an u8 value containing CAN FD flags (CANFD_BRS, CANFD_ESI)\n");
-	fprintf(stderr, "  <len>  is an u8 value containing the data length (0 .. 64)\n");
+	fprintf(stderr, "  <len>  is an u8 value containing the data length in hex (0 .. 40)\n");
 	fprintf(stderr, "  <can_data>  is always 64(!) u8 values containing the CAN FD frames data\n");
 	fprintf(stderr, "The max. four modifications are performed in the order AND -> OR -> XOR -> SET\n");
 	fprintf(stderr, "\n");
@@ -743,10 +743,18 @@ int main(int argc, char **argv)
 
 		case 's':
 			src_ifindex = if_nametoindex(optarg);
+			if (!src_ifindex) {
+				perror("src if_nametoindex");
+				exit(1);
+			}
 			break;
 
 		case 'd':
 			dst_ifindex = if_nametoindex(optarg);
+			if (!dst_ifindex) {
+				perror("dst if_nametoindex");
+				exit(1);
+			}
 			break;
 
 		case 'X':
@@ -766,7 +774,7 @@ int main(int argc, char **argv)
 			break;
 
 		case 'u':
-			uid = strtoul(optarg, (char **)NULL, 16);
+			uid = strtoul(optarg, NULL, 16);
 			break;
 
 		case 'l':
