@@ -87,6 +87,10 @@ void print_usage(char *prg)
                 "(Use 'i' for infinite loop - default: %d)\n", DEFAULT_LOOPS);
         fprintf(stderr, "         -t           (ignore timestamps: "
                 "send frames immediately)\n");
+        fprintf(stderr, "         -i           (interactive - wait "
+                "for ENTER key to process next frame)\n");
+	fprintf(stderr, "         -n <count>   (terminate after "
+		"processing <count> CAN frames)\n");
         fprintf(stderr, "         -g <ms>      (gap in milli "
                 "seconds - default: %d ms)\n", DEFAULT_GAP);
         fprintf(stderr, "         -s <s>       (skip gaps in "
@@ -139,7 +143,7 @@ static inline int frames_to_send(struct timeval *today, struct timeval *diff,
 	cmp.tv_sec  = log->tv_sec  + diff->tv_sec;
 	cmp.tv_usec = log->tv_usec + diff->tv_usec;
 
-	if (cmp.tv_usec > 1000000) {
+	if (cmp.tv_usec >= 1000000) {
 		cmp.tv_usec -= 1000000;
 		cmp.tv_sec++;
 	}
@@ -244,6 +248,8 @@ int main(int argc, char **argv)
 	FILE *infile = stdin;
 	unsigned long gap = DEFAULT_GAP; 
 	int use_timestamps = 1;
+	int interactive = 0; /* wait for ENTER keypress to process next frame */
+	int count = 0; /* end replay after sending count frames. 0 = disabled */
 	static int verbose, opt, delay_loops;
 	static unsigned long skipgap;
 	static int loopback_disable = 0;
@@ -254,7 +260,7 @@ int main(int argc, char **argv)
 	int eof, txmtu, i, j;
 	char *fret;
 
-	while ((opt = getopt(argc, argv, "I:l:tg:s:xv?")) != -1) {
+	while ((opt = getopt(argc, argv, "I:l:tin:g:s:xv?")) != -1) {
 		switch (opt) {
 		case 'I':
 			infile = fopen(optarg, "r");
@@ -278,6 +284,18 @@ int main(int argc, char **argv)
 			use_timestamps = 0;
 			break;
 
+		case 'i':
+			interactive = 1;
+			break;
+
+		case 'n':
+			count = atoi(optarg);
+			if (count < 1) {
+				print_usage(basename(argv[0]));
+				exit(1);
+			}
+			break;
+
 		case 'g':
 			gap = strtoul(optarg, NULL, 10);
 			break;
@@ -320,6 +338,12 @@ int main(int argc, char **argv)
 			printf("%d loops\n", loops);
 	}
 
+	/* ignore timestamps from logfile when in single step keypress mode */
+	if (interactive) {
+		use_timestamps = 0;
+		printf("interactive mode: press ENTER to process next CAN frame ...\n");
+	}
+
 	sleep_ts.tv_sec  =  gap / 1000;
 	sleep_ts.tv_nsec = (gap % 1000) * 1000000;
 
@@ -413,6 +437,10 @@ int main(int argc, char **argv)
 			while ((!use_timestamps) ||
 			       (frames_to_send(&today_tv, &diff_tv, &log_tv) < 0)) {
 
+				/* wait for keypress to process next frame */
+				if (interactive)
+					getchar();
+
 				/* log_tv/device/ascframe are valid here */
 
 				if (strlen(device) >= IFNAMSIZ) {
@@ -459,6 +487,9 @@ int main(int argc, char **argv)
 						else
 							fprint_long_canframe(stdout, &frame, "\n", CANLIB_VIEW_INDENT_SFF, CANFD_MAX_DLEN);
 					}
+
+					if (count && (--count == 0))
+						goto out;
 				}
 
 				/* read next non-comment frame from logfile */
