@@ -61,8 +61,10 @@ void print_usage(char *prg)
 	fprintf(stderr, "%s - send CAN-frames via CAN_RAW sockets.\n", prg);
 	fprintf(stderr, "\nUsage: %s <device> <can_frame>.\n", prg);
 	fprintf(stderr, "\n<can_frame>:\n");
-	fprintf(stderr, " <can_id>#{data}          for 'classic' CAN 2.0 data frames\n");
-	fprintf(stderr, " <can_id>#R{len}          for 'classic' CAN 2.0 data frames\n");
+	fprintf(stderr, " <can_id>#{data}          for Classical CAN 2.0 data frames\n");
+	fprintf(stderr, " <can_id>#R{len}          for Classical CAN 2.0 data frames\n");
+	fprintf(stderr, " <can_id>#{data}_{dlc}    for Classical CAN 2.0 data frames\n");
+	fprintf(stderr, " <can_id>#R{len}_{dlc}    for Classical CAN 2.0 data frames\n");
 	fprintf(stderr, " <can_id>##<flags>{data}  for CAN FD frames\n\n");
 	fprintf(stderr, "<can_id>:\n"
 	        " 3 (SFF) or 8 (EFF) hex chars\n");
@@ -70,11 +72,13 @@ void print_usage(char *prg)
 	        " 0..8 (0..64 CAN FD) ASCII hex-values (optionally separated by '.')\n");
 	fprintf(stderr, "{len}:\n"
 		 " an optional 0..8 value as RTR frames can contain a valid dlc field\n");
+	fprintf(stderr, "_{dlc}:\n"
+		 " an optional 9..F data length code value when payload length is 8\n");
 	fprintf(stderr, "<flags>:\n"
 	        " a single ASCII Hex value (0 .. F) which defines canfd_frame.flags\n\n");
 	fprintf(stderr, "Examples:\n");
 	fprintf(stderr, "  5A1#11.2233.44556677.88 / 123#DEADBEEF / 5AA# / 123##1 / 213##311223344 /\n"
-		 "  1F334455#1122334455667788 / 123#R / 00000123#R3\n\n");
+		 "  1F334455#1122334455667788_B / 123#R / 00000123#R3 / 333#R8_E\n\n");
 }
 
 
@@ -143,7 +147,7 @@ int main(int argc, char **argv)
 		}
 
 		/* ensure discrete CAN FD length values 0..8, 12, 16, 20, 24, 32, 64 */
-		frame.len = can_dlc2len(can_len2dlc(frame.len));
+		frame.len = can_fd_dlc2len(can_fd_len2dlc(frame.len));
 	}
 
 	/* disable default receive filter on this RAW socket */
